summaryrefslogtreecommitdiff
path: root/firmware
diff options
context:
space:
mode:
authorMichiel Van Der Kolk <not.valid@email.address>2005-03-28 00:00:24 +0000
committerMichiel Van Der Kolk <not.valid@email.address>2005-03-28 00:00:24 +0000
commit451dd48adc2ef29fd2f900693393cc9b9b4a849b (patch)
treee15d20e602261866617210bde007966ce9b19293 /firmware
parent853bc3dcf85aa1284a0e5b550277c40beb7697a9 (diff)
Sound api improvements, rockboy sound, contributed by xshock.
Playback of sound currently only works in boost mode, needs fixing. git-svn-id: svn://svn.rockbox.org/rockbox/trunk@6226 a1c6a512-1295-4272-9138-f99709370657
Diffstat (limited to 'firmware')
-rw-r--r--firmware/drivers/uda1380.c4
-rw-r--r--firmware/export/pcm_playback.h2
-rw-r--r--firmware/pcm_playback.c78
3 files changed, 71 insertions, 13 deletions
diff --git a/firmware/drivers/uda1380.c b/firmware/drivers/uda1380.c
index e8b8c14399..8c3cf61eae 100644
--- a/firmware/drivers/uda1380.c
+++ b/firmware/drivers/uda1380.c
@@ -49,7 +49,7 @@ unsigned short uda1380_defaults[2*NUM_DEFAULT_REGS] =
REG_I2S, I2S_IFMT_IIS,
REG_PWR, PON_PLL | PON_HP | PON_DAC | EN_AVC | PON_AVC | PON_BIAS,
REG_AMIX, AMIX_RIGHT(0x10) | AMIX_LEFT(0x10), /* 00=max, 3f=mute */
- REG_MASTER_VOL, MASTER_VOL_LEFT(0x7f) | MASTER_VOL_RIGHT(0x7f), /* 00=max, ff=mute */
+ REG_MASTER_VOL, MASTER_VOL_LEFT(0x20) | MASTER_VOL_RIGHT(0x20), /* 00=max, ff=mute */
REG_MIX_VOL, MIX_VOL_CHANNEL_1(0) | MIX_VOL_CHANNEL_2(0xff), /* 00=max, ff=mute */
REG_EQ, 0,
REG_MUTE, MUTE_CH2, /* Mute channel 2 (digital decimation filter) */
@@ -131,6 +131,8 @@ int uda1380_set_regs(void)
/* Initialize UDA1380 codec with default register values (uda1380_defaults) */
int uda1380_init(void)
{
+ PLLCR &= ~(1 << 22); /* Set AudioClk = FXTAL/2*/
+
if (uda1380_set_regs() == -1)
return -1;
diff --git a/firmware/export/pcm_playback.h b/firmware/export/pcm_playback.h
index c44fb283ec..6010293bbd 100644
--- a/firmware/export/pcm_playback.h
+++ b/firmware/export/pcm_playback.h
@@ -19,9 +19,11 @@
#ifndef PCM_PLAYBACK_H
#define PCM_PLAYBACK_H
+void pcm_set_frequency(unsigned int frequency);
void pcm_play_data(const unsigned char* start, int size,
void (*get_more)(unsigned char** start, long* size));
void pcm_play_stop(void);
bool pcm_is_playing(void);
+void pcm_set_volume(int volume);
#endif
diff --git a/firmware/pcm_playback.c b/firmware/pcm_playback.c
index fba85f083b..bc2218be69 100644
--- a/firmware/pcm_playback.c
+++ b/firmware/pcm_playback.c
@@ -37,20 +37,32 @@
#include "file.h"
#include "buffer.h"
-bool pcm_playing;
+#include "sprintf.h"
+#include "button.h"
+#include <string.h>
+
+static bool pcm_playing;
+static int pcm_freq = 0x6; // 44.1 in default
/* Set up the DMA transfer that kicks in when the audio FIFO gets empty */
-static void dma_start(const void *addr, long size)
-{
+static void dma_start(const void *addr_r, long size)
+{
pcm_playing = 1;
+ int i;
+
+ int align;
+ align = 4;
+
+ void* addr = (void*)((((unsigned int)addr_r) >> 2) << 2); // always align data, never pass unaligned data
+ size = (size >> 2) << 2; // size shoudl also be always multiple of 4
BUSMASTER_CTRL = 0x81; /* PARK[1,0]=10 + BCR24BIT */
/* Set up DMA transfer */
DIVR0 = 54; /* DMA0 is mapped into vector 54 in system.c */
- SAR0 = (unsigned long)addr; /* Source address */
+ SAR0 = ((unsigned long)addr) + align*4; /* Source address */
DAR0 = (unsigned long)&PDOR3; /* Destination address */
- BCR0 = size; /* Bytes to transfer */
+ BCR0 = size-(align*4); /* Bytes to transfer */
DMAROUTE = (DMAROUTE & 0xffffff00) | DMA0_REQ_AUDIO_1;
DMACONFIG = 1; /* Enable DMA0Req => set DMAROUTE |= DMA0_REQ_AUDIO_1 */
@@ -61,14 +73,10 @@ static void dma_start(const void *addr, long size)
ICR4 = (ICR4 & 0xffff00ff) | 0x00001c00;
IMR &= ~(1<<14); /* bit 14 is DMA0 */
- IIS2CONFIG = 0x4300; /* CLOCKSEL = AudioClk/8 (44.1kHz),
- data source = PDOR3 */
+ IIS2CONFIG = (pcm_freq << 12) | 0x300; /* CLOCKSEL for right frequency + data source = PDOR3 */
- PDOR3 = 0; /* These are needed to generate FIFO empty request to DMA.. */
- PDOR3 = 0;
- PDOR3 = 0;
- PDOR3 = 0;
- PDOR3 = 0;
+ for(i = 0; i < align; i++)
+ PDOR3 = ((unsigned int*)(addr))[i]; /* These are needed to generate FIFO empty request to DMA.. */
}
/* Stops the DMA transfer and interrupt */
@@ -76,12 +84,51 @@ static void dma_stop(void)
{
pcm_playing = 0;
DCR0 = 0;
+
+/* DMAROUTE &= 0xffffff00;
+ DMACONFIG = 0;*/
+
+ IIS2CONFIG = 0x800;
/* Disable DMA0 interrupt */
IMR |= (1<<14);
ICR4 &= 0xffff00ff;
}
+
+/* set volume of the main channel */
+void pcm_set_volume(int volume)
+{
+ if(volume > 0)
+ {
+ uda1380_mute(0);
+ uda1380_setvol(0xff - volume);
+ }
+ else
+ {
+ uda1380_mute(1);
+ }
+}
+
+/* sets frequency of input to DAC */
+void pcm_set_frequency(unsigned int frequency)
+{
+ switch(frequency)
+ {
+ case 11025:
+ pcm_freq = 0x2;
+ break;
+ case 22050:
+ pcm_freq = 0x4;
+ break;
+ case 44100:
+ pcm_freq = 0x6;
+ break;
+ default:
+ pcm_freq = 0x6;
+ }
+}
+
/* the registered callback function to ask for more mp3 data */
static void (*callback_for_more)(unsigned char**, long*) = NULL;
@@ -110,8 +157,15 @@ void DMA0(void)
unsigned char* start;
long size = 0;
+ int res = DSR0;
+
DSR0 = 1; /* Clear interrupt */
+ if(res == 0x41)
+ {
+ dma_stop();
+ }
+
if (callback_for_more)
{
callback_for_more(&start, &size);